/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2016 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.ui.internal.introspection.details.handler;

import java.util.Collection;
import java.util.List;

import org.conqat.ide.commons.ui.jface.TreeContentProviderBase;
import org.conqat.lib.commons.collections.Pair;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.TableLabelProviderBase;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.ScrolledComposite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeColumn;
import org.eclipse.ui.dialogs.PatternFilter;
import org.fortiss.tooling.kernel.extension.ITransformationProvider;
import org.fortiss.tooling.kernel.introspection.items.TransformationServiceIntrospectionDetailsItem;
import org.fortiss.tooling.kernel.service.ITransformationService;

/**
 * Introspection UI handler for the {@link ITransformationService}.
 * 
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating RED Hash:
 */
public final class TransformationServiceIntrospectionDetailsUIHandler extends
		IntrospectionDetailsUIHandlerBase {

	/** {@inheritDoc} */
	@Override
	public Control createComposite(ScrolledComposite parent) {
		String heading = "Type to serach registered transformation providers:";
		String footer =
				"Number of currently registered transformation providers: " +
						getInputObject().countHandlers();
		return createFilteredTreeInTabFolder(parent, heading, footer, "Registered Providers");
	}

	/** {@inheritDoc} */
	@Override
	protected void createTreeColumns(Tree tree) {
		TreeColumn col0 = new TreeColumn(tree, SWT.LEFT);
		col0.setText("Source / Target / Provider");
		col0.setWidth(250);

		TreeColumn col1 = new TreeColumn(tree, SWT.LEFT);
		col1.setText("Provider Class");
		col1.setWidth(400);

		TreeColumn col2 = new TreeColumn(tree, SWT.LEFT);
		col2.setText("Source EObject Class");
		col2.setWidth(400);

		TreeColumn col3 = new TreeColumn(tree, SWT.LEFT);
		col3.setText("Target EObject Class");
		col3.setWidth(400);
	}

	/** {@inheritDoc} */
	@Override
	protected ITreeContentProvider createContentProvider() {
		return new TreeContentProviderBase() {

			@Override
			public Object[] getChildren(Object parentElement) {
				if(parentElement == getInputObject()) {
					return getInputObject().getFirstHandlerKeyClasses().toArray();
				}
				if(parentElement instanceof Class<?>) {
					Class<?> firstClass = (Class<?>)parentElement;
					Collection<Class<?>> secondaryClasses =
							getInputObject().getSecondHandlerKeyClasses(firstClass);
					if(secondaryClasses == null || secondaryClasses.isEmpty()) {
						return new Object[0];
					}
					Pair<?, ?>[] pairs = new Pair<?, ?>[secondaryClasses.size()];
					int idx = 0;
					for(Class<?> secondary : secondaryClasses) {
						pairs[idx] = new Pair<Class<?>, Class<?>>(firstClass, secondary);
						idx++;
					}
					return pairs;
				}
				if(parentElement instanceof Pair) {
					Pair<?, ?> pair = (Pair<?, ?>)parentElement;
					if(pair.getFirst() instanceof Class) {
						Class<?> first = (Class<?>)pair.getFirst();
						Class<?> second = (Class<?>)pair.getSecond();
						List<ITransformationProvider> handlerList =
								getInputObject().getHandlerList(first, second);
						if(handlerList == null || handlerList.isEmpty()) {
							return new Object[0];
						}
						Pair<?, ?>[] pairs = new Pair<?, ?>[handlerList.size()];
						int idx = 0;
						for(ITransformationProvider provider : handlerList) {
							pairs[idx] =
									new Pair<ITransformationProvider, Pair<?, ?>>(provider, pair);
							idx++;
						}
						return pairs;
					}
					// no children for transformation providers
				}
				return new Object[0];
			}

		};
	}

	/** {@inheritDoc} */
	@Override
	protected ITableLabelProvider createLabelProvider() {
		return new TableLabelProviderBase() {
			@Override
			public String getColumnText(Object element, int columnIndex) {
				if(element instanceof Class) {
					return columnIndex == 0 ? ((Class<?>)element).getSimpleName() : "";
				}
				if(element instanceof Pair) {
					Pair<?, ?> pair = (Pair<?, ?>)element;
					if(pair.getFirst() instanceof Class) {
						Class<?> c = (Class<?>)((Pair<?, ?>)element).getSecond();
						return columnIndex == 0 ? c.getSimpleName() : "";
					}
					if(pair.getFirst() instanceof ITransformationProvider) {
						ITransformationProvider provider = (ITransformationProvider)pair.getFirst();
						Pair<?, ?> regClasses = (Pair<?, ?>)pair.getSecond();
						Class<?> sourceClass = (Class<?>)regClasses.getFirst();
						Class<?> targetClass = (Class<?>)regClasses.getSecond();
						switch(columnIndex) {
							case 0:
								return provider.getClass().getSimpleName();
							case 1:
								return provider.getClass().getName();
							case 2:
								return sourceClass.getName();
							case 3:
								return targetClass.getName();
						}
					}
				}
				return "";
			}
		};
	}

	/** {@inheritDoc} */
	@Override
	protected PatternFilter createPatternFilter() {
		return new TableViewerPatternFilter();
	}

	/** {@inheritDoc} */
	@Override
	protected TransformationServiceIntrospectionDetailsItem getInputObject() {
		return (TransformationServiceIntrospectionDetailsItem)dataItem;
	}
}
